@@ -119,7 +119,7 @@ gst_ring_buffer_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (G_OBJECT (ringbuffer));
 }
 
-static int linear_formats[4 * 2 * 2] = {
+static const int linear_formats[4 * 2 * 2] = {
   GST_S8,
   GST_S8,
   GST_U8,
@@ -138,7 +138,7 @@ static int linear_formats[4 * 2 * 2] = {
   GST_U32_BE
 };
 
-static int linear24_formats[3 * 2 * 2] = {
+static const int linear24_formats[3 * 2 * 2] = {
   GST_S24_3LE,
   GST_S24_3BE,
   GST_U24_3LE,
@@ -156,40 +156,45 @@ static int linear24_formats[3 * 2 * 2] = {
 static GstBufferFormat
 build_linear_format (int depth, int width, int unsignd, int big_endian)
 {
+  const gint *formats;
+
   if (width == 24) {
     switch (depth) {
       case 24:
-        depth = 0;
+        formats = &linear24_formats[0];
         break;
       case 20:
-        depth = 1;
+        formats = &linear24_formats[4];
         break;
       case 18:
-        depth = 2;
+        formats = &linear24_formats[8];
         break;
       default:
         return GST_UNKNOWN;
     }
-    return ((int (*)[2][2]) linear24_formats)[depth][!!unsignd][!!big_endian];
   } else {
     switch (depth) {
       case 8:
-        depth = 0;
+        formats = &linear_formats[0];
         break;
       case 16:
-        depth = 1;
+        formats = &linear_formats[4];
         break;
       case 24:
-        depth = 2;
+        formats = &linear_formats[8];
         break;
       case 32:
-        depth = 3;
+        formats = &linear_formats[12];
         break;
       default:
         return GST_UNKNOWN;
     }
   }
-  return ((int (*)[2][2]) linear_formats)[depth][!!unsignd][!!big_endian];
+  if (unsignd)
+    formats += 2;
+  if (big_endian)
+    formats += 1;
+  return (GstBufferFormat) * formats;
 }
 
 /**
@@ -649,6 +654,7 @@ gst_ring_buffer_release (GstRingBuffer * buf)
     res = rclass->release (buf);
 
   /* signal any waiters */
+  GST_DEBUG_OBJECT (buf, "signal waiter");
   GST_RING_BUFFER_SIGNAL (buf);
 
   if (!res)
@@ -815,9 +821,6 @@ gst_ring_buffer_pause_unlocked (GstRingBuffer * buf)
     goto done;
   }
 
-  /* signal any waiters */
-  GST_RING_BUFFER_SIGNAL (buf);
-
   rclass = GST_RING_BUFFER_GET_CLASS (buf);
   if (rclass->pause)
     res = rclass->pause (buf);
@@ -901,6 +904,7 @@ gst_ring_buffer_stop (GstRingBuffer * buf)
   }
 
   /* signal any waiters */
+  GST_DEBUG_OBJECT (buf, "signal waiter");
   GST_RING_BUFFER_SIGNAL (buf);
 
   rclass = GST_RING_BUFFER_GET_CLASS (buf);
@@ -1122,6 +1126,7 @@ gst_ring_buffer_commit (GstRingBuffer * buf, guint64 sample, guchar * data,
   gint segdone;
   gint segsize, segtotal, bps, sps;
   guint8 *dest;
+  guint to_write;
 
   g_return_val_if_fail (buf != NULL, -1);
   g_return_val_if_fail (buf->data != NULL, -1);
@@ -1133,8 +1138,9 @@ gst_ring_buffer_commit (GstRingBuffer * buf, guint64 sample, guchar * data,
   bps = buf->spec.bytes_per_sample;
   sps = buf->samples_per_seg;
 
+  to_write = len;
   /* write out all samples */
-  while (len > 0) {
+  while (to_write > 0) {
     gint sampleslen;
     gint writeseg, sampleoff;
 
@@ -1153,14 +1159,14 @@ gst_ring_buffer_commit (GstRingBuffer * buf, guint64 sample, guchar * data,
       diff = writeseg - segdone;
 
       GST_DEBUG
-          ("pointer at %d, sample %llu, write to %d-%d, len %d, diff %d, segtotal %d, segsize %d",
-          segdone, sample, writeseg, sampleoff, len, diff, segtotal, sps);
+          ("pointer at %d, sample %llu, write to %d-%d, to_write %d, diff %d, segtotal %d, segsize %d",
+          segdone, sample, writeseg, sampleoff, to_write, diff, segtotal, sps);
 
       /* segment too far ahead, we need to drop */
       if (diff < 0) {
         /* we need to drop one segment at a time, pretend we wrote a
          * segment. */
-        sampleslen = MIN (sps, len);
+        sampleslen = MIN (sps, to_write);
         goto next;
       }
 
@@ -1176,27 +1182,27 @@ gst_ring_buffer_commit (GstRingBuffer * buf, guint64 sample, guchar * data,
 
     /* we can write now */
     writeseg = writeseg % segtotal;
-    sampleslen = MIN (sps - sampleoff, len);
+    sampleslen = MIN (sps - sampleoff, to_write);
 
-    GST_DEBUG_OBJECT (buf, "write @%p seg %d, off %d, len %d",
+    GST_DEBUG_OBJECT (buf, "write @%p seg %d, off %d, sampleslen %d",
         dest + writeseg * segsize, writeseg, sampleoff, sampleslen);
 
     memcpy (dest + (writeseg * segsize) + (sampleoff * bps), data,
         (sampleslen * bps));
 
   next:
-    len -= sampleslen;
+    to_write -= sampleslen;
     sample += sampleslen;
     data += sampleslen * bps;
   }
 
-  return len;
+  return len - to_write;
 
   /* ERRORS */
 not_started:
   {
     GST_DEBUG_OBJECT (buf, "stopped processing");
-    return -1;
+    return len - to_write;
   }
 }
 
