@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) <2005> Philippe Khalaf <burger@speedy.org> 
+ * Copyright (C) <2005> Nokia Corporation <kai.vehmanen@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -173,12 +174,13 @@ static gboolean
 gst_base_rtp_depayload_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstBaseRTPDepayload *filter;
+  GstBaseRTPDepayloadClass *bclass;
 
   filter = GST_BASE_RTP_DEPAYLOAD (gst_pad_get_parent (pad));
   g_return_val_if_fail (filter != NULL, FALSE);
   g_return_val_if_fail (GST_IS_BASE_RTP_DEPAYLOAD (filter), FALSE);
 
-  GstBaseRTPDepayloadClass *bclass = GST_BASE_RTP_DEPAYLOAD_GET_CLASS (filter);
+  bclass = GST_BASE_RTP_DEPAYLOAD_GET_CLASS (filter);
 
   if (bclass->set_caps)
     return bclass->set_caps (filter, caps);
@@ -219,14 +221,22 @@ gst_base_rtp_depayload_handle_sink_event (GstPad * pad, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:
     {
-      GST_DEBUG_OBJECT (filter,
-          "Upstream sent a NEWSEGMENT, handle in worker thread.");
-      /* the worker thread will assign a new RTP-TS<->GST-TS mapping
-       * based on the next processed RTP packet */
-      filter->need_newsegment = TRUE;
-      gst_event_unref (event);
-      break;
+      /* intercept NEWSEGMENT events only if the packet scheduler thread
+         is active */
+      if (filter->thread) {
+        GST_DEBUG_OBJECT (filter,
+            "Upstream sent a NEWSEGMENT, handle in worker thread.");
+        /* the worker thread will assign a new RTP-TS<->GST-TS mapping
+         * based on the next processed RTP packet */
+        filter->need_newsegment = TRUE;
+        gst_event_unref (event);
+        break;
+      } else {
+        GST_DEBUG_OBJECT (filter,
+            "Upstream sent a NEWSEGMENT, passing through.");
+      }
     }
+      /* note: pass through to default if no thread running */
     default:
       /* pass other events forward */
       res = gst_pad_push_event (filter->srcpad, event);
@@ -241,6 +251,7 @@ gst_base_rtp_depayload_add_to_queue (GstBaseRTPDepayload * filter,
     GstBuffer * in)
 {
   GQueue *queue = filter->queue;
+  int i;
 
   /* our first packet, just push it */
   QUEUE_LOCK (filter);
@@ -255,7 +266,7 @@ gst_base_rtp_depayload_add_to_queue (GstBaseRTPDepayload * filter,
     queueseq = gst_rtp_buffer_get_seq (GST_BUFFER (g_queue_peek_head (queue)));
 
     /* look for right place to insert it */
-    int i = 0;
+    i = 0;
 
     while (seqnum > queueseq) {
       gpointer data;
@@ -350,6 +361,8 @@ gst_base_rtp_depayload_queue_release (GstBaseRTPDepayload * filter)
   GQueue *queue = filter->queue;
   guint32 headts, tailts;
   GstBaseRTPDepayloadClass *bclass;
+  gfloat q_size_secs;
+  guint maxtsunits;
 
   if (g_queue_is_empty (queue))
     return;
@@ -357,10 +370,10 @@ gst_base_rtp_depayload_queue_release (GstBaseRTPDepayload * filter)
   /* if our queue is getting to big (more than RTP_QUEUEDELAY ms of data)
    * release heading buffers
    */
-  GST_DEBUG_OBJECT (filter, "clockrate %d, queue_delay %d", filter->clock_rate,
-      filter->queue_delay);
-  gfloat q_size_secs = (gfloat) filter->queue_delay / 1000;
-  guint maxtsunits = (gfloat) filter->clock_rate * q_size_secs;
+  /*GST_DEBUG_OBJECT (filter, "clockrate %d, queue_delay %d", filter->clock_rate,
+     filter->queue_delay); */
+  q_size_secs = (gfloat) filter->queue_delay / 1000;
+  maxtsunits = (gfloat) filter->clock_rate * q_size_secs;
 
   QUEUE_LOCK (filter);
   headts =
@@ -402,11 +415,15 @@ gst_base_rtp_depayload_thread (GstBaseRTPDepayload * filter)
 static gboolean
 gst_base_rtp_depayload_start_thread (GstBaseRTPDepayload * filter)
 {
-  GST_DEBUG_OBJECT (filter, "Starting queue release thread");
-  filter->thread_running = TRUE;
-  filter->thread = g_thread_create ((GThreadFunc) gst_base_rtp_depayload_thread,
-      filter, TRUE, NULL);
-  GST_DEBUG_OBJECT (filter, "Started queue release thread");
+  /* only launch the thread if processing is needed */
+  if (filter->queue_delay) {
+    GST_DEBUG_OBJECT (filter, "Starting queue release thread");
+    filter->thread_running = TRUE;
+    filter->thread =
+        g_thread_create ((GThreadFunc) gst_base_rtp_depayload_thread, filter,
+        TRUE, NULL);
+    GST_DEBUG_OBJECT (filter, "Started queue release thread");
+  }
   return TRUE;
 }
 
