@@ -19,6 +19,20 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gsttagid3
+ * @short_description: tag mappings and support functions for plugins
+ *                     dealing with ID3v1 and ID3v2 tags
+ * @see_also: #GstTagList
+ * 
+ * <refsect2>
+ * <para>
+ * Contains various utility functions for plugins to parse or create
+ * ID3 tags and map ID3v2 identifiers to and from GStreamer identifiers.
+ * </para>
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -189,6 +203,7 @@ static GstTagEntryMatch tag_matches[] = {
   {GST_TAG_COMMENT, "COMM"},
   {GST_TAG_ALBUM_VOLUME_NUMBER, "TPOS"},
   {GST_TAG_DURATION, "TLEN"},
+  {GST_TAG_ISRC, "TSRC"},
   {NULL, NULL}
 };
 
@@ -209,11 +224,15 @@ gst_tag_from_id3_tag (const gchar * id3_tag)
 
   while (tag_matches[i].gstreamer_tag != NULL) {
     if (strncmp (id3_tag, tag_matches[i].original_tag, 5) == 0) {
-      break;
+      return tag_matches[i].gstreamer_tag;
     }
     i++;
   }
-  return tag_matches[i].gstreamer_tag;
+
+  GST_INFO ("Cannot map ID3v2 tag '%c%c%c%c' to GStreamer tag",
+      id3_tag[0], id3_tag[1], id3_tag[2], id3_tag[3]);
+
+  return NULL;
 }
 
 /**
