@@ -65,6 +65,7 @@ struct _GstPlayBin
   GstElement *audio_sink;
   GstElement *video_sink;
   GstElement *visualisation;
+  GstElement *pending_visualisation;
   GstElement *volume_element;
   GstElement *textoverlay_element;
   gfloat volume;
@@ -220,6 +221,7 @@ gst_play_bin_init (GstPlayBin * play_bin)
   play_bin->video_sink = NULL;
   play_bin->audio_sink = NULL;
   play_bin->visualisation = NULL;
+  play_bin->pending_visualisation = NULL;
   play_bin->volume_element = NULL;
   play_bin->textoverlay_element = NULL;
   play_bin->volume = 1.0;
@@ -258,12 +260,130 @@ gst_play_bin_dispose (GObject * object)
     gst_object_unref (play_bin->visualisation);
     play_bin->visualisation = NULL;
   }
+  if (play_bin->pending_visualisation != NULL) {
+    gst_element_set_state (play_bin->pending_visualisation, GST_STATE_NULL);
+    gst_object_unref (play_bin->pending_visualisation);
+    play_bin->pending_visualisation = NULL;
+  }
+  if (play_bin->textoverlay_element != NULL) {
+    gst_object_unref (play_bin->textoverlay_element);
+    play_bin->textoverlay_element = NULL;
+  }
   g_free (play_bin->font_desc);
   play_bin->font_desc = NULL;
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+static void
+gst_play_bin_vis_unblocked (GstPad * tee_pad, gboolean blocked,
+    gpointer user_data)
+{
+  /* Unblocked */
+}
+
+static void
+gst_play_bin_vis_blocked (GstPad * tee_pad, gboolean blocked,
+    gpointer user_data)
+{
+  GstPlayBin *play_bin = GST_PLAY_BIN (user_data);
+  GstBin *vis_bin = NULL;
+  GstPad *vis_sink_pad = NULL, *vis_src_pad = NULL, *vqueue_pad = NULL;
+  GstState bin_state;
+
+  /* We want to disable visualisation */
+  if (!GST_IS_ELEMENT (play_bin->pending_visualisation)) {
+    /* Set visualisation element to READY */
+    gst_element_set_state (play_bin->visualisation, GST_STATE_READY);
+    goto beach;
+  }
+
+  vis_bin =
+      GST_BIN (gst_object_get_parent (GST_OBJECT (play_bin->visualisation)));
+
+  if (!GST_IS_BIN (vis_bin) || !GST_IS_PAD (tee_pad)) {
+    goto beach;
+  }
+
+  vis_src_pad = gst_element_get_pad (play_bin->visualisation, "src");
+  vis_sink_pad = gst_pad_get_peer (tee_pad);
+
+  /* Can be fakesink */
+  if (GST_IS_PAD (vis_src_pad)) {
+    vqueue_pad = gst_pad_get_peer (vis_src_pad);
+  }
+
+  if (!GST_IS_PAD (vis_sink_pad)) {
+    goto beach;
+  }
+
+  /* Check the bin's state */
+  GST_OBJECT_LOCK (vis_bin);
+  bin_state = GST_STATE (vis_bin);
+  GST_OBJECT_UNLOCK (vis_bin);
+
+  /* Unlink */
+  gst_pad_unlink (tee_pad, vis_sink_pad);
+  gst_object_unref (vis_sink_pad);
+  vis_sink_pad = NULL;
+
+  if (GST_IS_PAD (vqueue_pad)) {
+    gst_pad_unlink (vis_src_pad, vqueue_pad);
+    gst_object_unref (vis_src_pad);
+    vis_src_pad = NULL;
+  }
+
+  /* Remove from vis_bin */
+  gst_bin_remove (vis_bin, play_bin->visualisation);
+  /* Set state to NULL */
+  gst_element_set_state (play_bin->visualisation, GST_STATE_NULL);
+  /* And loose our ref */
+  gst_object_unref (play_bin->visualisation);
+
+  if (play_bin->pending_visualisation) {
+    /* Ref this new visualisation element before adding to the bin */
+    gst_object_ref (play_bin->pending_visualisation);
+    /* Add the new one */
+    gst_bin_add (vis_bin, play_bin->pending_visualisation);
+    /* Synchronizing state */
+    gst_element_set_state (play_bin->pending_visualisation, bin_state);
+
+    vis_sink_pad = gst_element_get_pad (play_bin->pending_visualisation,
+        "sink");
+    vis_src_pad = gst_element_get_pad (play_bin->pending_visualisation, "src");
+
+    if (!GST_IS_PAD (vis_sink_pad) || !GST_IS_PAD (vis_src_pad)) {
+      goto beach;
+    }
+
+    /* Link */
+    gst_pad_link (tee_pad, vis_sink_pad);
+    gst_pad_link (vis_src_pad, vqueue_pad);
+  }
+
+  /* We are done */
+  gst_object_unref (play_bin->visualisation);
+  play_bin->visualisation = play_bin->pending_visualisation;
+  play_bin->pending_visualisation = NULL;
+
+beach:
+  if (vis_sink_pad) {
+    gst_object_unref (vis_sink_pad);
+  }
+  if (vis_src_pad) {
+    gst_object_unref (vis_src_pad);
+  }
+  if (vqueue_pad) {
+    gst_object_unref (vqueue_pad);
+  }
+  if (vis_bin) {
+    gst_object_unref (vis_bin);
+  }
+
+  /* Unblock the pad */
+  gst_pad_set_blocked_async (tee_pad, FALSE, gst_play_bin_vis_unblocked,
+      play_bin);
+}
 
 static void
 gst_play_bin_set_property (GObject * object, guint prop_id,
@@ -302,15 +422,70 @@ gst_play_bin_set_property (GObject * object, guint prop_id,
       g_hash_table_remove (play_bin->cache, "abin");
       break;
     case ARG_VIS_PLUGIN:
-      if (play_bin->visualisation != NULL) {
-        gst_object_unref (play_bin->visualisation);
-      }
-      play_bin->visualisation = g_value_get_object (value);
-      if (play_bin->visualisation != NULL) {
-        gst_object_ref (play_bin->visualisation);
-        gst_object_sink (GST_OBJECT (play_bin->visualisation));
+    {
+      /* Do we already have a visualisation change pending ? */
+      if (play_bin->pending_visualisation) {
+        gst_object_unref (play_bin->pending_visualisation);
+        play_bin->pending_visualisation = g_value_get_object (value);
+        /* Take ownership */
+        if (play_bin->pending_visualisation) {
+          gst_object_ref (play_bin->pending_visualisation);
+          gst_object_sink (GST_OBJECT (play_bin->pending_visualisation));
+        }
+      } else {
+        play_bin->pending_visualisation = g_value_get_object (value);
+
+        /* Take ownership */
+        if (play_bin->pending_visualisation) {
+          gst_object_ref (play_bin->pending_visualisation);
+          gst_object_sink (GST_OBJECT (play_bin->pending_visualisation));
+        }
+
+        /* Was there a visualisation already set ? */
+        if (play_bin->visualisation != NULL) {
+          GstBin *vis_bin = NULL;
+
+          vis_bin =
+              GST_BIN (gst_object_get_parent (GST_OBJECT (play_bin->
+                      visualisation)));
+
+          /* Check if the visualisation is already in a bin */
+          if (GST_IS_BIN (vis_bin)) {
+            GstPad *vis_sink_pad = NULL, *tee_pad = NULL;
+
+            /* Now get tee pad and block it async */
+            vis_sink_pad = gst_element_get_pad (play_bin->visualisation,
+                "sink");
+            if (!GST_IS_PAD (vis_sink_pad)) {
+              goto beach;
+            }
+            tee_pad = gst_pad_get_peer (vis_sink_pad);
+            if (!GST_IS_PAD (tee_pad)) {
+              goto beach;
+            }
+
+            /* Block with callback */
+            gst_pad_set_blocked_async (tee_pad, TRUE, gst_play_bin_vis_blocked,
+                play_bin);
+          beach:
+            if (vis_sink_pad) {
+              gst_object_unref (vis_sink_pad);
+            }
+            if (tee_pad) {
+              gst_object_unref (tee_pad);
+            }
+            gst_object_unref (vis_bin);
+          } else {
+            play_bin->visualisation = play_bin->pending_visualisation;
+            play_bin->pending_visualisation = NULL;
+          }
+        } else {
+          play_bin->visualisation = play_bin->pending_visualisation;
+          play_bin->pending_visualisation = NULL;
+        }
       }
       break;
+    }
     case ARG_VOLUME:
       play_bin->volume = g_value_get_double (value);
       if (play_bin->volume_element) {
@@ -486,39 +661,57 @@ gen_text_element (GstPlayBin * play_bin)
   GstElement *element, *csp, *overlay, *vbin;
   GstPad *pad;
 
+  /* Create our bin */
+  element = gst_bin_new ("textbin");
+
+  /* Text overlay */
   overlay = gst_element_factory_make ("textoverlay", "overlay");
-  g_object_set (G_OBJECT (overlay),
-      "halign", "center", "valign", "bottom", NULL);
-  play_bin->textoverlay_element = overlay;
-  if (play_bin->font_desc) {
-    g_object_set (G_OBJECT (play_bin->textoverlay_element),
-        "font-desc", play_bin->font_desc, NULL);
-  }
+
+  /* Create the video rendering bin */
   vbin = gen_video_element (play_bin);
+
+  /* If no overlay return the video bin */
   if (!overlay) {
-    g_warning ("No overlay (pango) element, subtitles disabled");
+    GST_WARNING ("No overlay (pango) element, subtitles disabled");
     return vbin;
   }
+
+  /* Set some parameters */
+  g_object_set (G_OBJECT (overlay),
+      "halign", "center", "valign", "bottom", NULL);
+  if (play_bin->font_desc) {
+    g_object_set (G_OBJECT (overlay), "font-desc", play_bin->font_desc, NULL);
+  }
+
+  /* Take a ref */
+  play_bin->textoverlay_element = GST_ELEMENT (gst_object_ref (overlay));
+
   csp = gst_element_factory_make ("ffmpegcolorspace", "subtitlecsp");
-  element = gst_bin_new ("textbin");
-  gst_element_link_many (csp, overlay, vbin, NULL);
+
+  /* Add our elements */
   gst_bin_add_many (GST_BIN (element), csp, overlay, vbin, NULL);
 
+  /* Link */
+  gst_element_link_pads (csp, "src", overlay, "video_sink");
+  gst_element_link_pads (overlay, "src", vbin, "sink");
+
+  /* Add ghost pads on the subtitle bin */
   pad = gst_element_get_pad (overlay, "text_sink");
-#define gst_element_add_ghost_pad(element, pad, name) \
-    gst_element_add_pad (element, gst_ghost_pad_new (name, pad))
-  gst_element_add_ghost_pad (element, pad, "text_sink");
+  gst_element_add_pad (element, gst_ghost_pad_new ("text_sink", pad));
   gst_object_unref (pad);
 
   pad = gst_element_get_pad (csp, "sink");
-  gst_element_add_ghost_pad (element, pad, "sink");
+  gst_element_add_pad (element, gst_ghost_pad_new ("sink", pad));
   gst_object_unref (pad);
 
+  /* Set state to READY */
+  gst_element_set_state (element, GST_STATE_READY);
+
   return element;
 }
 
 /* make the element (bin) that contains the elements needed to perform
- * audio playback. 
+ * audio playback.
  *
  *  +-------------------------------------------------------------+
  *  | abin                                                        |
@@ -528,7 +721,7 @@ gen_text_element (GstPlayBin * play_bin)
  *  |   |  +---------+   +----------+   +---------+   +---------+ |
  * sink-+                                                         |
  *  +-------------------------------------------------------------+
- *                  
+ *
  */
 static GstElement *
 gen_audio_element (GstPlayBin * play_bin)
@@ -580,7 +773,7 @@ gen_audio_element (GstPlayBin * play_bin)
   gst_element_link_pads (volume, "src", sink, "sink");
 
   pad = gst_element_get_pad (conv, "sink");
-  gst_element_add_ghost_pad (element, pad, "sink");
+  gst_element_add_pad (element, gst_ghost_pad_new ("sink", pad));
   gst_object_unref (pad);
 
   gst_element_set_state (element, GST_STATE_READY);
@@ -628,15 +821,21 @@ gen_vis_element (GstPlayBin * play_bin)
   GstElement *vqueue, *aqueue;
   GstPad *pad, *rpad;
 
+  asink = gen_audio_element (play_bin);
+  if (!asink)
+    return NULL;
+  vsink = gen_video_element (play_bin);
+  if (!vsink) {
+    gst_object_unref (asink);
+    return NULL;
+  }
+
   element = gst_bin_new ("visbin");
   tee = gst_element_factory_make ("tee", "tee");
 
   vqueue = gst_element_factory_make ("queue", "vqueue");
   aqueue = gst_element_factory_make ("queue", "aqueue");
 
-  asink = gen_audio_element (play_bin);
-  vsink = gen_video_element (play_bin);
-
   gst_bin_add (GST_BIN (element), asink);
   gst_bin_add (GST_BIN (element), vqueue);
   gst_bin_add (GST_BIN (element), aqueue);
@@ -673,7 +872,7 @@ gen_vis_element (GstPlayBin * play_bin)
   gst_object_unref (pad);
 
   pad = gst_element_get_pad (tee, "sink");
-  gst_element_add_ghost_pad (element, pad, "sink");
+  gst_element_add_pad (element, gst_ghost_pad_new ("sink", pad));
   gst_object_unref (pad);
 
   return element;
@@ -765,7 +964,10 @@ remove_sinks (GstPlayBin * play_bin)
     play_bin->frame = NULL;
   }
 
-  play_bin->textoverlay_element = NULL;
+  if (play_bin->textoverlay_element) {
+    gst_object_unref (play_bin->textoverlay_element);
+    play_bin->textoverlay_element = NULL;
+  }
 }
 
 /* loop over the streams and set up the pipeline to play this
@@ -778,13 +980,15 @@ remove_sinks (GstPlayBin * play_bin)
  * one can switch the streams.
  */
 static gboolean
-add_sink (GstPlayBin * play_bin, GstElement * sink, GstPad * srcpad)
+add_sink (GstPlayBin * play_bin, GstElement * sink, GstPad * srcpad,
+    GstPad * subtitle_pad)
 {
   GstPad *sinkpad;
   GstPadLinkReturn linkres;
   GstElement *parent;
   GstStateChangeReturn stateret;
 
+  g_return_val_if_fail (sink != NULL, FALSE);
   /* this is only for debugging */
   parent = gst_pad_get_parent_element (srcpad);
   if (parent) {
@@ -810,8 +1014,19 @@ add_sink (GstPlayBin * play_bin, GstElement * sink, GstPad * srcpad)
   if (GST_PAD_LINK_FAILED (linkres))
     goto link_failed;
 
+  if (GST_IS_PAD (subtitle_pad)) {
+    sinkpad = gst_element_get_pad (sink, "text_sink");
+    linkres = gst_pad_link (subtitle_pad, sinkpad);
+    gst_object_unref (sinkpad);
+  }
+
+  /* try to link the subtitle pad of the sink to the stream */
+  if (GST_PAD_LINK_FAILED (linkres)) {
+    goto subtitle_failed;
+  }
+
   /* we got the sink succesfully linked, now keep the sink
-   * in out internal list */
+   * in our internal list */
   play_bin->sinks = g_list_prepend (play_bin->sinks, sink);
 
   return TRUE;
@@ -840,6 +1055,22 @@ link_failed:
     gst_bin_remove (GST_BIN (play_bin), sink);
     return FALSE;
   }
+subtitle_failed:
+  {
+    gchar *capsstr;
+    GstCaps *caps;
+
+    /* could not link this stream */
+    caps = gst_pad_get_caps (subtitle_pad);
+    capsstr = gst_caps_to_string (caps);
+    GST_DEBUG_OBJECT (play_bin,
+        "subtitle link failed when adding sink, caps %s, reason %d", capsstr,
+        linkres);
+    g_free (capsstr);
+    g_free (caps);
+
+    return TRUE;
+  }
 }
 
 static gboolean
@@ -849,7 +1080,7 @@ setup_sinks (GstPlayBaseBin * play_base_bin, GstPlayBaseGroup * group)
   GList *streaminfo = NULL, *s;
   gboolean need_vis = FALSE;
   gboolean need_text = FALSE;
-  GstPad *textsrcpad = NULL, *textsinkpad = NULL, *pad;
+  GstPad *textsrcpad = NULL, *pad = NULL;
   GstElement *sink;
   gboolean res = TRUE;
 
@@ -857,7 +1088,7 @@ setup_sinks (GstPlayBaseBin * play_base_bin, GstPlayBaseGroup * group)
   if (play_bin->sinks) {
     remove_sinks (play_bin);
   }
-  GST_DEBUG ("setupsinks");
+  GST_DEBUG_OBJECT (play_base_bin, "setupsinks");
 
   /* find out what to do */
   if (group->type[GST_STREAM_TYPE_VIDEO - 1].npads > 0 &&
@@ -887,30 +1118,75 @@ setup_sinks (GstPlayBaseBin * play_base_bin, GstPlayBaseGroup * group)
     } else {
       sink = gen_audio_element (play_bin);
     }
+    if (!sink)
+      return FALSE;
     pad = gst_element_get_pad (group->type[GST_STREAM_TYPE_AUDIO - 1].preroll,
         "src");
-    res = add_sink (play_bin, sink, pad);
+    res = add_sink (play_bin, sink, pad, NULL);
     gst_object_unref (pad);
   }
 
   /* link video */
   if (group->type[GST_STREAM_TYPE_VIDEO - 1].npads > 0) {
     if (need_text) {
-      sink = gen_text_element (play_bin);
+      GstObject *parent = NULL, *grandparent = NULL;
+      GstPad *ghost = NULL;
 
-      textsinkpad = gst_element_get_pad (sink, "text_sink");
+      sink = gen_text_element (play_bin);
       textsrcpad =
           gst_element_get_pad (group->type[GST_STREAM_TYPE_TEXT - 1].preroll,
           "src");
-      gst_pad_link (textsrcpad, textsinkpad);
-      gst_object_unref (textsinkpad);
-      gst_object_unref (textsrcpad);
+      /* This pad is from subtitle-bin, we need to create a ghost pad to have
+         common grandparents */
+      parent = gst_object_get_parent (GST_OBJECT (textsrcpad));
+      if (!parent) {
+        GST_WARNING_OBJECT (textsrcpad, "subtitle pad has no parent !");
+        gst_object_unref (textsrcpad);
+        textsrcpad = NULL;
+        goto beach;
+      }
+
+      grandparent = gst_object_get_parent (parent);
+      if (!grandparent) {
+        GST_WARNING_OBJECT (textsrcpad, "subtitle pad has no grandparent !");
+        gst_object_unref (parent);
+        gst_object_unref (textsrcpad);
+        textsrcpad = NULL;
+        goto beach;
+      }
+
+      ghost = gst_ghost_pad_new ("text_src", textsrcpad);
+      if (!GST_IS_PAD (ghost)) {
+        GST_WARNING_OBJECT (textsrcpad, "failed creating ghost pad for "
+            "subtitle-bin");
+        gst_object_unref (parent);
+        gst_object_unref (grandparent);
+        gst_object_unref (textsrcpad);
+        textsrcpad = NULL;
+        goto beach;
+      }
+
+      if (gst_element_add_pad (GST_ELEMENT (grandparent), ghost)) {
+        gst_object_unref (textsrcpad);
+        textsrcpad = ghost;
+      } else {
+        GST_WARNING_OBJECT (ghost, "failed adding ghost pad on subtitle-bin");
+        gst_object_unref (ghost);
+        gst_object_unref (textsrcpad);
+        textsrcpad = NULL;
+      }
+
+      gst_object_unref (parent);
+      gst_object_unref (grandparent);
     } else {
       sink = gen_video_element (play_bin);
     }
+  beach:
+    if (!sink)
+      return FALSE;
     pad = gst_element_get_pad (group->type[GST_STREAM_TYPE_VIDEO - 1].preroll,
         "src");
-    res = add_sink (play_bin, sink, pad);
+    res = add_sink (play_bin, sink, pad, textsrcpad);
     gst_object_unref (pad);
   }
 
@@ -1072,6 +1348,12 @@ plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (gst_play_bin_debug, "playbin", 0, "play bin");
 
+#ifdef ENABLE_NLS
+  GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
+      LOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif /* ENABLE_NLS */
+
   return gst_element_register (plugin, "playbin", GST_RANK_NONE,
       GST_TYPE_PLAY_BIN);
 }
