@@ -18,21 +18,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:element-tcpclientsink
- * @see_also: #tcpclientsrc
- *
- * <refsect2>
- * <title>Example launch line</title>
- * |[
- * # server:
- * nc -l -p 3000
- * # client:
- * gst-launch fdsrc fd=1 ! tcpclientsink protocol=none port=3000
- * ]| everything you type in the client is shown on the server
- * </refsect2>
- */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -42,6 +27,13 @@
 #include "gsttcpclientsink.h"
 #include <string.h>             /* memset */
 
+/* elementfactory information */
+static GstElementDetails gst_tcp_client_sink_details =
+GST_ELEMENT_DETAILS ("TCP Client sink",
+    "Sink/Network",
+    "Send data as a client over the network via TCP",
+    "Thomas Vander Stichele <thomas at apestaart dot org>");
+
 /* TCPClientSink signals and args */
 enum
 {
@@ -50,7 +42,7 @@ enum
   LAST_SIGNAL
 };
 
-GST_DEBUG_CATEGORY_STATIC (tcpclientsink_debug);
+GST_DEBUG_CATEGORY (tcpclientsink_debug);
 #define GST_CAT_DEFAULT (tcpclientsink_debug)
 
 enum
@@ -121,12 +113,10 @@ gst_tcp_client_sink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class, &sinktemplate);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sinktemplate));
 
-  gst_element_class_set_details_simple (element_class,
-      "TCP client sink", "Sink/Network",
-      "Send data as a client over the network via TCP",
-      "Thomas Vander Stichele <thomas at apestaart dot org>");
+  gst_element_class_set_details (element_class, &gst_tcp_client_sink_details);
 }
 
 static void
@@ -140,23 +130,21 @@ gst_tcp_client_sink_class_init (GstTCPClientSink * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasesink_class = (GstBaseSinkClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
+  parent_class = g_type_class_ref (GST_TYPE_BASE_SINK);
 
   gobject_class->set_property = gst_tcp_client_sink_set_property;
   gobject_class->get_property = gst_tcp_client_sink_get_property;
   gobject_class->finalize = gst_tcp_client_sink_finalize;
 
-  g_object_class_install_property (gobject_class, ARG_HOST,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_HOST,
       g_param_spec_string ("host", "Host", "The host/IP to send the packets to",
-          TCP_DEFAULT_HOST, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_PORT,
+          TCP_DEFAULT_HOST, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PORT,
       g_param_spec_int ("port", "Port", "The port to send the packets to",
-          0, TCP_HIGHEST_PORT, TCP_DEFAULT_PORT,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          0, TCP_HIGHEST_PORT, TCP_DEFAULT_PORT, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_PROTOCOL,
       g_param_spec_enum ("protocol", "Protocol", "The protocol to wrap data in",
-          GST_TYPE_TCP_PROTOCOL, GST_TCP_PROTOCOL_NONE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_TYPE_TCP_PROTOCOL, GST_TCP_PROTOCOL_NONE, G_PARAM_READWRITE));
 
   gstelement_class->change_state = gst_tcp_client_sink_change_state;
 
@@ -172,7 +160,7 @@ gst_tcp_client_sink_init (GstTCPClientSink * this)
   this->host = g_strdup (TCP_DEFAULT_HOST);
   this->port = TCP_DEFAULT_PORT;
 
-  this->sock_fd.fd = -1;
+  this->sock_fd = -1;
   this->protocol = GST_TCP_PROTOCOL_NONE;
   GST_OBJECT_FLAG_UNSET (this, GST_TCP_CLIENT_SINK_OPEN);
 }
@@ -183,8 +171,6 @@ gst_tcp_client_sink_finalize (GObject * gobject)
   GstTCPClientSink *this = GST_TCP_CLIENT_SINK (gobject);
 
   g_free (this->host);
-
-  G_OBJECT_CLASS (parent_class)->finalize (gobject);
 }
 
 static gboolean
@@ -210,8 +196,8 @@ gst_tcp_client_sink_setcaps (GstBaseSink * bsink, GstCaps * caps)
         GST_DEBUG_OBJECT (sink, "Sending caps %s through GDP", string);
         g_free (string);
 
-        if (!gst_tcp_gdp_write_caps (GST_ELEMENT (sink), sink->sock_fd.fd,
-                caps, TRUE, sink->host, sink->port))
+        if (!gst_tcp_gdp_write_caps (GST_ELEMENT (sink), sink->sock_fd, caps,
+                TRUE, sink->host, sink->port))
           goto gdp_write_error;
 
         sink->caps_sent = TRUE;
@@ -253,7 +239,7 @@ gst_tcp_client_sink_render (GstBaseSink * bsink, GstBuffer * buf)
       break;
     case GST_TCP_PROTOCOL_GDP:
       GST_LOG_OBJECT (sink, "Sending buffer header through GDP");
-      if (!gst_tcp_gdp_write_buffer (GST_ELEMENT (sink), sink->sock_fd.fd, buf,
+      if (!gst_tcp_gdp_write_buffer (GST_ELEMENT (sink), sink->sock_fd, buf,
               TRUE, sink->host, sink->port))
         goto gdp_write_error;
       break;
@@ -262,7 +248,7 @@ gst_tcp_client_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   }
 
   /* write buffer data */
-  wrote = gst_tcp_socket_write (sink->sock_fd.fd, GST_BUFFER_DATA (buf), size);
+  wrote = gst_tcp_socket_write (sink->sock_fd, GST_BUFFER_DATA (buf), size);
 
   if (wrote < size)
     goto write_error;
@@ -280,7 +266,7 @@ write_error:
   {
     GST_ELEMENT_ERROR (sink, RESOURCE, WRITE,
         (_("Error while sending data to \"%s:%d\"."), sink->host, sink->port),
-        ("Only %" G_GSIZE_FORMAT " of %u bytes written: %s",
+        ("Only %d of %d bytes written: %s",
             wrote, GST_BUFFER_SIZE (buf), g_strerror (errno)));
     return GST_FLOW_ERROR;
   }
@@ -360,12 +346,12 @@ gst_tcp_client_sink_start (GstTCPClientSink * this)
   /* create sending client socket */
   GST_DEBUG_OBJECT (this, "opening sending client socket to %s:%d", this->host,
       this->port);
-  if ((this->sock_fd.fd = socket (AF_INET, SOCK_STREAM, 0)) == -1) {
+  if ((this->sock_fd = socket (AF_INET, SOCK_STREAM, 0)) == -1) {
     GST_ELEMENT_ERROR (this, RESOURCE, OPEN_WRITE, (NULL), GST_ERROR_SYSTEM);
     return FALSE;
   }
   GST_DEBUG_OBJECT (this, "opened sending client socket with fd %d",
-      this->sock_fd.fd);
+      this->sock_fd);
 
   /* look up name if we need to */
   ip = gst_tcp_host_to_ip (GST_ELEMENT (this), this->host);
@@ -383,7 +369,7 @@ gst_tcp_client_sink_start (GstTCPClientSink * this)
   g_free (ip);
 
   GST_DEBUG_OBJECT (this, "connecting to server");
-  ret = connect (this->sock_fd.fd, (struct sockaddr *) &this->server_sin,
+  ret = connect (this->sock_fd, (struct sockaddr *) &this->server_sin,
       sizeof (this->server_sin));
 
   if (ret) {
@@ -417,7 +403,10 @@ gst_tcp_client_sink_stop (GstTCPClientSink * this)
   if (!GST_OBJECT_FLAG_IS_SET (this, GST_TCP_CLIENT_SINK_OPEN))
     return TRUE;
 
-  gst_tcp_socket_close (&this->sock_fd);
+  if (this->sock_fd != -1) {
+    close (this->sock_fd);
+    this->sock_fd = -1;
+  }
 
   GST_OBJECT_FLAG_UNSET (this, GST_TCP_CLIENT_SINK_OPEN);
 
@@ -436,7 +425,7 @@ gst_tcp_client_sink_change_state (GstElement * element,
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      if (!gst_tcp_client_sink_start (sink))
+      if (!gst_tcp_client_sink_start (GST_TCP_CLIENT_SINK (element)))
         goto start_failure;
       break;
     default:
@@ -446,7 +435,7 @@ gst_tcp_client_sink_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_NULL:
-      gst_tcp_client_sink_stop (sink);
+      gst_tcp_client_sink_stop (GST_TCP_CLIENT_SINK (element));
     default:
       break;
   }
